
/* Includes ------------------------------------------------------------------*/
#include "fsl_iocon.h"
#include "hal_lpc54608.h"
#include "adc_manager.h"



/* Macros --------------------------------------------------------------------*/

/*
 * Check port and pin, update "gpio" struct
 */
#define __hal_lpc54608_get_gpio_data(port, pin, gpio)       \
    do {                                                    \
        if (get_gpio_data(port, pin, gpio) == HAL_GPIO_NOK) \
        {                                                   \
            /* Invalid port or pin. */                      \
            return HAL_GPIO_NOK;                            \
        }                                                   \
    } while (0)

/* Fields --------------------------------------------------------------------*/

static uint8_t initialized = 0;
//static ADC_Type adcType;

/* Private functions ---------------------------------------------------------*/

static int32_t get_gpio_data(int32_t port, int32_t pin, hal_lpc54608_gpio_data* gpio)
{
  // set default value of analog converters
  gpio->adc_rank = HAL_GPIO_NOK;
    if (port == HAL_LPC54608_GPIO_PORT_MCU)
    {
        /*
         * MCU virtual port: convert port and pin using real MCU port and pin
         *
         * MCU port = (pin / 32) + 1
         * MCU pin = pin % 32
         */

        if (pin < 0)
        {
            /* Invalid pin. */
            return HAL_GPIO_NOK;
        }
        return hal_lpc54608_get_gpio_data((pin >> 5) + 1, pin & 0x1f, gpio);
    }
    else
    {
        return hal_lpc54608_get_gpio_data(port, pin, gpio);
    }
}

/* API -----------------------------------------------------------------------*/

int32_t LLHAL_GPIO_IMPL_setPinMode(int32_t port, int32_t pin, int32_t mode)
{
    /* Initialize HAL only once. */
    if (!initialized)
    {
        hal_lpc54608_initialize();
       
          /*    adc initialization */

        if (adc_manager_initialize() != HAL_GPIO_OK)
        		{
        			// unknown error
        			return HAL_GPIO_NOK;
        		}

        initialized = !initialized;
    }

    /* Retrieve GPIO data. */
    hal_lpc54608_gpio_data gpio;
    __hal_lpc54608_get_gpio_data(port, pin, &gpio);

    /* Initialize GPIO config structure. */
    gpio_pin_config_t config; 
    config.outputLogic = 0;
    

    /* Initialize IOCON config structure. */
    uint32_t modefunc;
    modefunc = IOCON_FUNC0;
    
     switch (mode)
    {
    case LLHAL_GPIO_MODE_DIGITAL_INPUT:
    	config.pinDirection = kGPIO_DigitalInput;
        modefunc |= IOCON_MODE_INACT | IOCON_DIGITAL_EN;
        break;

    case LLHAL_GPIO_MODE_DIGITAL_INPUT_PULLUP:
        config.pinDirection = kGPIO_DigitalInput;
        modefunc |= IOCON_MODE_PULLUP | IOCON_DIGITAL_EN;
        break;

    case LLHAL_GPIO_MODE_DIGITAL_OUTPUT:
        config.pinDirection = kGPIO_DigitalOutput;
        modefunc |= IOCON_MODE_INACT | IOCON_DIGITAL_EN;
        break;

    case LLHAL_GPIO_MODE_ANALOG_INPUT:

    	if (gpio.adc_rank == HAL_GPIO_NOK)
    	{
    		// this is not a ADC pin
    		return HAL_GPIO_NOK;
    	}

        config.pinDirection = kGPIO_DigitalInput;
        modefunc |= IOCON_MODE_INACT | IOCON_ANALOG_EN | IOCON_INPFILT_OFF;
        break;

    case LLHAL_GPIO_MODE_ANALOG_OUTPUT:

        config.pinDirection = kGPIO_DigitalOutput;
        modefunc |= IOCON_MODE_PULLUP | IOCON_ANALOG_EN;
        break;

    default:
        /* Invalid mode. */
        return HAL_GPIO_NOK;
    }

    /* Initialize GPIO. */
    GPIO_PinInit(GPIO, (uint32_t) gpio.port, (uint32_t) gpio.pin, &config);

    /* Configure IOCON. */
    IOCON_PinMuxSet(IOCON, (uint8_t) gpio.port, (uint8_t) gpio.pin, modefunc);

    /* No error. */
    return HAL_GPIO_OK;
}

int32_t LLHAL_GPIO_IMPL_getDigitalValue(int32_t port, int32_t pin)
{
    /* Retrieve GPIO data. */
    hal_lpc54608_gpio_data gpio;
    __hal_lpc54608_get_gpio_data(port, pin, &gpio);

    return GPIO_ReadPinInput(GPIO, (uint32_t) gpio.port, (uint32_t) gpio.pin) == 1U ? LLHAL_GPIO_DIGITAL_HIGH : LLHAL_GPIO_DIGITAL_LOW;
}

int32_t LLHAL_GPIO_IMPL_setDigitalValue(int32_t port, int32_t pin, int32_t value)
{
    /* Retrieve GPIO data. */
    hal_lpc54608_gpio_data gpio;
    __hal_lpc54608_get_gpio_data(port, pin, &gpio);

    GPIO_WritePinOutput(GPIO, (uint32_t) gpio.port, (uint32_t) gpio.pin, value == LLHAL_GPIO_DIGITAL_LOW ? 0U : 1U);
    return HAL_GPIO_OK;
}

int32_t LLHAL_GPIO_IMPL_getAnalogValue(int32_t port, int32_t pin)
{
	 hal_lpc54608_gpio_data gpio;
	 __hal_lpc54608_get_gpio_data(port, pin, &gpio);
	 if (gpio.adc_rank == HAL_GPIO_NOK)
	 {
           // this is not an ADC pin
           return HAL_GPIO_NOK;
	 }
         printf("analog input test version \n");
	 return adc_manager_get_value(&gpio);
	
            

}

int32_t LLHAL_GPIO_IMPL_setAnalogValue(int32_t port, int32_t pin, int32_t percentage)
{
    /*  No unused pins connected on Arduino ports are connected to any pwm output. */
    return HAL_GPIO_NOK;
}

/* EOF -----------------------------------------------------------------------*/

