/*
 * C
 *
 * Copyright 2015-2016 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
#include <LLNET_SSL_CONTEXT_impl.h>
#include <LLNET_SSL_CONSTANTS.h>
#include <LLNET_SSL_ERRORS.h>
#include <wolfssl/ssl.h>
#include <stdlib.h>
#include <wolfssl/internal.h>
#include <wolfssl/error-ssl.h>
#ifdef __cplusplus
	extern "C" {
#endif

/*external function*/
extern int32_t LLNET_SSL_TranslateReturnCode(int32_t wolfSSL_error);

int32_t LLNET_SSL_CONTEXT_IMPL_createClientContext(int32_t protocol, uint8_t retry){
	LLNET_SSL_DEBUG_TRACE("%s(method=%d)\n", __func__, protocol);
	WOLFSSL_CTX* ctx = NULL;
	switch (protocol) {

		case SSLv3_PROTOCOL:
			ctx = wolfSSL_CTX_new(wolfSSLv3_client_method());
			break;
		case TLSv1_PROTOCOL:
			ctx = wolfSSL_CTX_new(wolfTLSv1_client_method());
			break;
		case TLSv1_1_PROTOCOL:
			ctx = wolfSSL_CTX_new(wolfTLSv1_1_client_method());
			break;
		case TLSv1_2_PROTOCOL:
			ctx = wolfSSL_CTX_new(wolfTLSv1_2_client_method());
			break;
		default:
			break;
	}
	LLNET_SSL_DEBUG_TRACE("%s(method=%d) return ctx=%p\n", __func__, protocol,ctx);
	if(ctx != NULL){
		return (int32_t)ctx;
	}
	return J_CREATE_SSL_CONTEXT_ERROR; //error
}


int32_t LLNET_SSL_CONTEXT_IMPL_addTrustedCert(int32_t context, int8_t *cert, int32_t off, int32_t len, int32_t format, uint8_t retry){

	int32_t encoding;
	if(format == CERT_PEM_FORMAT){
		encoding = SSL_FILETYPE_PEM;
	}else if(format == CERT_DER_FORMAT){
		encoding = SSL_FILETYPE_ASN1;
	}else{
		//unsupported encoded format
		return J_BAD_ENCODED_CERT_FORMAT;
	}

	//load the certificate as trusted certificate
	int32_t ret =  wolfSSL_CTX_load_verify_buffer((WOLFSSL_CTX*) context, (uint8_t*)(cert+off), len, encoding);
	return LLNET_SSL_TranslateReturnCode(ret);
}


int32_t LLNET_SSL_CONTEXT_IMPL_clearTrustStore(int32_t context, uint8_t retry){

	//clear all trusted CA certificates from the context
	int32_t ret = wolfSSL_CTX_UnloadCAs((WOLFSSL_CTX*) context) ;
	return LLNET_SSL_TranslateReturnCode(ret);
}

int32_t LLNET_SSL_CONTEXT_IMPL_closeContext(int32_t context, uint8_t retry){
	LLNET_SSL_DEBUG_TRACE("%s(context=%p)\n", __func__, (WOLFSSL_CTX*) context);
	wolfSSL_CTX_free((WOLFSSL_CTX*) context);
	return J_SSL_NO_ERROR;
}
#ifdef __cplusplus
	}
#endif
