/*
 * C
 *
 * Copyright 2015-2017 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
#include <LLNET_SSL_X509_CERT_impl.h>
#include <LLNET_SSL_CONSTANTS.h>
#include <LLNET_SSL_ERRORS.h>
#include <wolfssl/ssl.h>
#include <wolfssl/internal.h>
#include <stdlib.h>
#include <sni.h>

#ifdef __cplusplus
	extern "C" {
#endif

static int32_t LLNET_SSL_X509_CERT_IMPL_parseCert(WOLFSSL_CERT_MANAGER* cm, int8_t* buff,
                                   int32_t len, int32_t format);

int32_t LLNET_SSL_X509_CERT_IMPL_parse(int8_t *cert, int32_t off, int32_t len, uint8_t retry){

	int32_t format = J_CERT_PARSE_ERROR;
	WOLFSSL_CERT_MANAGER* cm = wolfSSL_CertManagerNew();

	if(cm){
		//try to parse as PEM certificate
		if(LLNET_SSL_X509_CERT_IMPL_parseCert(cm, (int8_t*) (cert+off), len, SSL_FILETYPE_PEM) == 0){
			//enconded PEM certificate
			format =  CERT_PEM_FORMAT;
		}
		//try to parse as DER certificate
		else if(LLNET_SSL_X509_CERT_IMPL_parseCert(cm, (int8_t*) (cert+off), len, SSL_FILETYPE_ASN1) == 0){
			//enconded DER certificate
			format = CERT_DER_FORMAT;
		}

		//free cert manager
		wolfSSL_CertManagerFree(cm);
	}
	return format;
}


static int32_t LLNET_SSL_X509_CERT_IMPL_parseCert(WOLFSSL_CERT_MANAGER* cm, int8_t* certBuf,
                                   int32_t len, int32_t format)
{
    int ret = 0;
		buffer der;
		der.buffer = NULL;
		der.length = 0;
#ifdef WOLFSSL_SMALL_STACK
		DecodedCert* cert = NULL;
#else
		DecodedCert  cert[1];
#endif
#ifdef WOLFSSL_SMALL_STACK
			cert = (DecodedCert*)XMALLOC(sizeof(DecodedCert), cm->heap, DYNAMIC_TYPE_TMP_BUFFER);
			if (cert == NULL){
				return J_MEMORY_ERROR; //memory error
			}
#endif                                                    

		if (format == SSL_FILETYPE_PEM) {
			int eccKey = 0;
#ifdef WOLFSSL_SMALL_STACK
			EncryptedInfo* info = NULL;
#else
			EncryptedInfo  info[1];
#endif
#ifdef WOLFSSL_SMALL_STACK
			info = (EncryptedInfo*)XMALLOC(sizeof(EncryptedInfo), cm->heap, DYNAMIC_TYPE_TMP_BUFFER);
			if (info == NULL) {
					XFREE(cert, cm->heap, DYNAMIC_TYPE_CERT);
					return J_MEMORY_ERROR; //memory error
			}
#endif  
				
			info->set      = 0;
			info->ctx      = NULL;
			info->consumed = 0;
			//convert to encoded der format
			ret = PemToDer((const byte*)certBuf, len, CERT_TYPE, &der, cm->heap, info, &eccKey);
			if (ret == 0){
					InitDecodedCert(cert, der.buffer, der.length, cm->heap);
			}
#ifdef WOLFSSL_SMALL_STACK
			XFREE(info, cm->heap, DYNAMIC_TYPE_CERT);
#endif
		}
		else if(format == SSL_FILETYPE_ASN1){
			InitDecodedCert(cert, (byte*)certBuf, len, cm->heap);
		}else{
#ifdef WOLFSSL_SMALL_STACK
			XFREE(cert, cm->heap, DYNAMIC_TYPE_CERT);
#endif
			return J_BAD_ENCODED_CERT_FORMAT; //unsupported format
		}

		if (ret == 0){
			//just parse the certificate, no verification
			ret = ParseCertRelative(cert, CERT_TYPE, 0, cm); 
		}

		if (ret == 0){
			FreeDecodedCert(cert);
		}

		XFREE(der.buffer, cm->heap, DYNAMIC_TYPE_CERT);
#ifdef WOLFSSL_SMALL_STACK
		XFREE(cert, cm->heap, DYNAMIC_TYPE_CERT);
#endif
		return ret;
}


#if defined(USER_TIME)

#include <time.h>
// function used to retrieve current time
extern int64_t LLMJVM_IMPL_getCurrentTime__Z(uint8_t);

/*
 * Custom XTIME() function implementation to get ticks since the APOCH
 * Used to check certificate validity
 */
time_t XTIME(time_t * timer){
	//get the application time in seconds
	time_t time =  (time_t)(LLMJVM_IMPL_getCurrentTime__Z(0) / 1000);
	if(timer){
		*timer = time;
	}
	return time;
}
#endif

#ifdef __cplusplus
	}
#endif
