/*
 * C
 *
 * Copyright 2017 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
/*
 * Compatibility file for platform projects based on IAR EWARM 8.x and MicroEJ 4.1 architectures (compiled with IAR EWARM 7.x).
 */
#include "iar7_compatibility.h"
#include "math.h"
#include "stdio.h"

#if __ICCARM__ == 0
  #error This compatibility file has not been tested with your current IAR architecture.
#endif

#if __IAR_SYSTEMS_ICC__ == 9
  // IAR 7.x->8.x compatibility 
  // NaN test functions have changed:
  //    __iar_FDtest => __iar_FPclassify32
  //    __iar_Dtest => __iar_FPclassify64
  //    
#if __DOUBLE_SIZE__ != 8
#error This compatibility file has not been tested with your current IAR architecture.
#endif

  // Put back macros defined in IAR7 header file: arm/inc/c/ymath.h
  /* MACROS FOR _X_FNAME(Dtest) RETURN (0 => ZERO) */
  #define IAR7_DENORM         (-2)    /* C9X only */
  #define IAR7_FINITE         (-1)
  #define IAR7_INFCODE        1
  #define IAR7_NANCODE        2

  int iar7NanConvert(int iar8Code){
    switch(iar8Code){
    case FP_INFINITE:  return IAR7_INFCODE;
    case FP_NAN:       return IAR7_NANCODE;
    case FP_NORMAL:    return IAR7_FINITE;
    case FP_SUBNORMAL: return IAR7_DENORM;
    case FP_ZERO:      return 0;
    default: while(1);
    }
  }
 
  int __iar_FDtest(float x){
    return iar7NanConvert(__iar_FPclassify32(x));
  }

  int __iar_Dtest(double x){
    return iar7NanConvert(__iar_FPclassify64(x));
  }
#elif __IAR_SYSTEMS_ICC__ == 8
  // IAR 7.x 
  // this file base version
#else
  // Unknown version
  #error This compatibility file has not been tested with your current IAR ICC version.
#endif

// Test functions
// call iar7_compatibility_check() at the beginning of the main function.

extern char iceTea_lang_Float___isNaN(float f);
extern char iceTea_lang_Double___isNaN(double f);

void iar7_error(){ 
  printf("IAR 7 compatibility check error (see file %s).\n", __FILE__); 
  while(1); 
}

static void testf(float f, int expected){
  if(iceTea_lang_Float___isNaN(f) != expected){
    iar7_error();
  }
}

static void testd(double d, int expected){
  if(iceTea_lang_Double___isNaN(d) != expected){
    iar7_error();
  }
}

static void testNaN(){
  testf(1.0f, 0);  
  testf(fmod(1,0), 1);
  testd((double)1.0, 0);  
  testd((double)NAN, 1);
}

void iar7_compatibility_check(){
  testNaN();
}
