/*
 * Copyright (c) 2013 - 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "fsl_device_registers.h"
#include "fsl_debug_console.h"
#include "FreeRTOS.h"
#include "task.h"
#include "board.h"
#include "cpuload.h"

#include "pin_mux.h"
#include <stdbool.h>

#include "microjvm_main.h"
#include "iar7_compatibility.h"

/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define MICROJVM_STACK_SIZE      (12 * 1024)
#define JAVA_TASK_PRIORITY       (11)                       /** Should be > tskIDLE_PRIORITY & < configTIMER_TASK_PRIORITY */
#define JAVA_TASK_STACK_SIZE     (MICROJVM_STACK_SIZE / 4)

/*******************************************************************************
 * External functions declarations
 ******************************************************************************/

/*******************************************************************************
 * Code
 ******************************************************************************/

/*!
 * @brief Java task function that runs in an OS task
 */
void xJavaTaskFunction(void * pvParameters)
{
    cpuload_init();
    //LLCOMM_stack_initialize();
    microjvm_main();
    vTaskDelete(xTaskGetCurrentTaskHandle());
}

/*!
 * @brief Main function
 */
int main(void)
{
    /* Init board hardware. */

    /* Configure main clock to max = 180 MHz. */
    BOARD_BootClockPLL180M();

    /* Perform pin initialization. */
    BOARD_InitPins();

    /* Perform LCD initialization. */
    BOARD_InitLCD();

    /* Enable the on-chip flash prefetch. */
    BOARD_FlashEnablePrefetch();

    /* Initialize the debug console. */
    BOARD_InitDebugConsole();

    /* Initialize the external SDRAM. */
    BOARD_InitSDRAM();

    /* Initialize the ENET. */
    BOARD_InitENET();

    /* Initialize the SDIF module. */
    BOARD_InitSDIF();

    /* Initialize the SPIFI module. */
    BOARD_InitSPIFI();

    PRINTF("Board init finished.\r\n");

    /* Check compatibility between IAR ICC version and MicroEJ 4.1 architecture */
    iar7_compatibility_check();
    
    /* Create the MicroJvm task. */
    xTaskCreate(xJavaTaskFunction, "MicroJvm", JAVA_TASK_STACK_SIZE, NULL, JAVA_TASK_PRIORITY, NULL);
    vTaskStartScheduler();
}
