/*
 * Copyright (c) 2016, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "fsl_common.h"
#include "fsl_iocon.h"
#include "pin_mux.h"
#include "fsl_pint.h"

void BOARD_InitPins(void)
{
    /* Enable clock for IOCON module. */
    CLOCK_EnableClock(kCLOCK_Iocon);
    CLOCK_EnableClock(kCLOCK_InputMux);

    /* Initialize the PINT module. */
    PINT_Init(PINT);
}

void BOARD_InitUsartPins(void)
{
#if !defined(UNIT_TEST) && !defined(COREMARK_TEST) && defined(EXTERNAL_TRACES)
    uint32_t usart_traces_switch = (uint32_t)(& _usart_traces_switch);
#else
    uint32_t usart_traces_switch = USART_TRACES_USB;
#endif

  if (usart_traces_switch == USART_TRACES_USB) {
      /* USART0 pin configuration. */
      IOCON_PinMuxSet(IOCON, 0, 29, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* FC0_RXD */
      IOCON_PinMuxSet(IOCON, 0, 30, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* FC0_TXD */
  } else if (usart_traces_switch == USART_TRACES_EXT) {
      /* USART4 pin configuration. */
      IOCON_PinMuxSet(IOCON, 3, 26, IOCON_MODE_INACT | IOCON_FUNC3 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* FC4_RXD */
      IOCON_PinMuxSet(IOCON, 3, 27, IOCON_MODE_INACT | IOCON_FUNC3 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* FC4_TXD */
  }
}

void BOARD_InitSdramPins(void)
{
    /* EMC SDRAM pin configuration. */
    IOCON_PinMuxSet(IOCON, 0, 18, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A0 */
    IOCON_PinMuxSet(IOCON, 0, 19, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A1 */
    IOCON_PinMuxSet(IOCON, 0, 20, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A2 */
    IOCON_PinMuxSet(IOCON, 0, 21, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A3 */
    IOCON_PinMuxSet(IOCON, 1,  5, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A4 */
    IOCON_PinMuxSet(IOCON, 1,  6, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A5 */
    IOCON_PinMuxSet(IOCON, 1,  7, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A6 */
    IOCON_PinMuxSet(IOCON, 1,  8, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A7 */
    IOCON_PinMuxSet(IOCON, 1, 26, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A8 */
    IOCON_PinMuxSet(IOCON, 1, 27, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A9 */
    IOCON_PinMuxSet(IOCON, 1, 16, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A10 */
    IOCON_PinMuxSet(IOCON, 1, 23, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A11 */
    IOCON_PinMuxSet(IOCON, 1, 24, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A12 */
    IOCON_PinMuxSet(IOCON, 1, 25, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A13 */
    IOCON_PinMuxSet(IOCON, 3, 25, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_A14 */

    IOCON_PinMuxSet(IOCON, 0,  2, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D0 */
    IOCON_PinMuxSet(IOCON, 0,  3, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D1 */
    IOCON_PinMuxSet(IOCON, 0,  4, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D2 */
    IOCON_PinMuxSet(IOCON, 0,  5, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D3 */
    IOCON_PinMuxSet(IOCON, 0,  6, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D4 */
    IOCON_PinMuxSet(IOCON, 0,  7, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D5 */
    IOCON_PinMuxSet(IOCON, 0,  8, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D6 */
    IOCON_PinMuxSet(IOCON, 0,  9, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D7 */
    IOCON_PinMuxSet(IOCON, 1, 19, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D8 */
    IOCON_PinMuxSet(IOCON, 1, 20, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D9 */
    IOCON_PinMuxSet(IOCON, 1, 21, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D10 */
    IOCON_PinMuxSet(IOCON, 1,  4, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D11 */
    IOCON_PinMuxSet(IOCON, 1, 28, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D12 */
    IOCON_PinMuxSet(IOCON, 1, 29, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D13 */
    IOCON_PinMuxSet(IOCON, 1, 30, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D14 */
    IOCON_PinMuxSet(IOCON, 1, 31, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_D15 */

    IOCON_PinMuxSet(IOCON, 1,  9, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_CASn */
    IOCON_PinMuxSet(IOCON, 1, 10, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_RASn */
    IOCON_PinMuxSet(IOCON, 1, 11, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_CLK0 */
    IOCON_PinMuxSet(IOCON, 1, 12, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_DYCSn0 */
    IOCON_PinMuxSet(IOCON, 1, 13, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_DQM0 */
    IOCON_PinMuxSet(IOCON, 1, 14, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_DQM1 */
    IOCON_PinMuxSet(IOCON, 1, 15, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_CKE0 */
    IOCON_PinMuxSet(IOCON, 0, 15, IOCON_MODE_INACT | IOCON_FASTI2C_EN | IOCON_FUNC6 | IOCON_DIGITAL_EN); /* EMC_WEn */
}

void BOARD_InitLcdPins(void)
{
    /* LCD pin configuration. */
    IOCON_PinMuxSet(IOCON, 2, 21, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD3 */
    IOCON_PinMuxSet(IOCON, 2, 22, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD4 */
    IOCON_PinMuxSet(IOCON, 2, 23, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD5 */
    IOCON_PinMuxSet(IOCON, 2, 24, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD6 */
    IOCON_PinMuxSet(IOCON, 2, 25, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD7 */

    IOCON_PinMuxSet(IOCON, 2, 28, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD10 */
    IOCON_PinMuxSet(IOCON, 2, 29, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD11 */
    IOCON_PinMuxSet(IOCON, 2, 30, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD12 */
    IOCON_PinMuxSet(IOCON, 2, 31, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD13 */
    IOCON_PinMuxSet(IOCON, 3,  0, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD14 */
    IOCON_PinMuxSet(IOCON, 3,  1, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD15 */

    IOCON_PinMuxSet(IOCON, 3,  5, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD19 */
    IOCON_PinMuxSet(IOCON, 3,  6, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD20 */
    IOCON_PinMuxSet(IOCON, 3,  7, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD21 */
    IOCON_PinMuxSet(IOCON, 3,  8, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD22 */
    IOCON_PinMuxSet(IOCON, 3,  9, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_VD23 */

    IOCON_PinMuxSet(IOCON, 2, 13, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_ANALOG_EN  | IOCON_INPFILT_OFF); /* LCD_DCLK */
    IOCON_PinMuxSet(IOCON, 2, 11, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_PWR */
    IOCON_PinMuxSet(IOCON, 2, 16, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_LP */
    IOCON_PinMuxSet(IOCON, 2, 14, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_FP */
    IOCON_PinMuxSet(IOCON, 2, 15, IOCON_MODE_INACT | IOCON_FUNC1 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD_AC_ENAB_M */

    IOCON_PinMuxSet(IOCON, 3, 31, IOCON_MODE_INACT | IOCON_FUNC2 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* LCD Backlight */
}

void BOARD_InitI2CPins(void)
{
    /* I2C pin configuration. */
    IOCON_PinMuxSet(IOCON, 3, 23, IOCON_FUNC1 | IOCON_MODE_PULLUP | IOCON_DIGITAL_EN); /* FC2_SDAX */
    IOCON_PinMuxSet(IOCON, 3, 24, IOCON_FUNC1 | IOCON_MODE_PULLUP | IOCON_DIGITAL_EN); /* FC2_SCLX */
}

void BOARD_InitTouchPins(void)
{
    /* Touch controller pin configuration. */
    IOCON_PinMuxSet(IOCON, 2, 27, IOCON_FUNC0 | IOCON_DIGITAL_EN);                     /* CT_RSTn */
    IOCON_PinMuxSet(IOCON, 4,  0, IOCON_FUNC5 | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF); /* CT_INTR */
}

void BOARD_InitButtonPins(void)
{
    /* User button pin configuration. The pins for ISP/User buttons SW2(0,6), SW3(0,5), SW4(0,4) are used by the EMC. */
    IOCON_PinMuxSet(IOCON, BOARD_SW5_GPIO_PORT, BOARD_SW5_GPIO_PIN, IOCON_FUNC0 | IOCON_MODE_PULLUP | IOCON_GPIO_MODE | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF);
}

void BOARD_InitLedPins(void)
{
    /* User LEDs pin configuration. */
    IOCON_PinMuxSet(IOCON, BOARD_LED1_GPIO_PORT, BOARD_LED1_GPIO_PIN, IOCON_FUNC0 | IOCON_GPIO_MODE | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF);
    IOCON_PinMuxSet(IOCON, BOARD_LED3_GPIO_PORT, BOARD_LED3_GPIO_PIN, IOCON_FUNC0 | IOCON_GPIO_MODE | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF);
    IOCON_PinMuxSet(IOCON, BOARD_LED2_GPIO_PORT, BOARD_LED2_GPIO_PIN, IOCON_FUNC0 | IOCON_GPIO_MODE | IOCON_DIGITAL_EN | IOCON_INPFILT_OFF);
}

void BOARD_InitSpifiPins(void)
{
    /* SPIFI pin configuration. */
    IOCON_PinMuxSet(IOCON, 0, 23, IOCON_FUNC6 | IOCON_MODE_PULLUP | IOCON_DIGITAL_EN); /* SPIFI_CSn */
    IOCON_PinMuxSet(IOCON, 0, 24, IOCON_FUNC6 | IOCON_MODE_PULLUP | IOCON_DIGITAL_EN); /* SPIFI_IO0 */
    IOCON_PinMuxSet(IOCON, 0, 25, IOCON_FUNC6 | IOCON_MODE_PULLUP | IOCON_DIGITAL_EN); /* SPIFI_IO1 */
    IOCON_PinMuxSet(IOCON, 0, 26, IOCON_FUNC6 | IOCON_MODE_PULLUP | IOCON_DIGITAL_EN); /* SPIFI_CLK */
    IOCON_PinMuxSet(IOCON, 0, 27, IOCON_FUNC6 | IOCON_MODE_PULLUP | IOCON_DIGITAL_EN); /* SPIFI_IO3 */
    IOCON_PinMuxSet(IOCON, 0, 28, IOCON_FUNC6 | IOCON_MODE_PULLUP | IOCON_DIGITAL_EN); /* SPIFI_IO2 */
}

#if SPIFI_ENABLE_RECOVERY_FEATURE
void BOARD_ReconfigureSpifiPinsForRecovery(void)
{
    /* SPIFI pin re-configuration, needed for the recovery feature. */
    IOCON_PinMuxSet(IOCON, 0, 23, IOCON_FUNC0 | IOCON_MODE_INACT | IOCON_DIGITAL_EN); /* SPIFI_CSn */
    IOCON_PinMuxSet(IOCON, 0, 24, IOCON_FUNC0 | IOCON_MODE_INACT | IOCON_DIGITAL_EN); /* SPIFI_IO0 */
    IOCON_PinMuxSet(IOCON, 0, 26, IOCON_FUNC0 | IOCON_MODE_INACT | IOCON_DIGITAL_EN); /* SPIFI_CLK */
    IOCON_PinMuxSet(IOCON, 0, 27, IOCON_FUNC0 | IOCON_MODE_INACT | IOCON_DIGITAL_EN); /* SPIFI_IO3 */
}
#endif

void BOARD_InitEnetPins(void)
{
    uint32_t slew_mask = 0;

    /* If ENET clock (which is the same as core clock) is higher than 96MHz, configure the pins in fast mode. */
    if (CLOCK_GetFreq(kCLOCK_CoreSysClk) > 96000000U)
    {
        slew_mask = IOCON_PIO_SLEW_MASK;
    }

    /* ENET pin configuration. */
    IOCON_PinMuxSet(IOCON, 4,  8, IOCON_FUNC1 | slew_mask | IOCON_MODE_INACT | IOCON_DIGITAL_EN); /* ENET_TXD0 */
    IOCON_PinMuxSet(IOCON, 4, 10, IOCON_FUNC1 | slew_mask | IOCON_MODE_INACT | IOCON_DIGITAL_EN); /* ENET_CRS_DV */
    IOCON_PinMuxSet(IOCON, 4, 11, IOCON_FUNC1 | slew_mask | IOCON_MODE_INACT | IOCON_DIGITAL_EN); /* ENET_RXD0 */
    IOCON_PinMuxSet(IOCON, 4, 12, IOCON_FUNC1 | slew_mask | IOCON_MODE_INACT | IOCON_DIGITAL_EN); /* ENET_RXD1 */
    IOCON_PinMuxSet(IOCON, 4, 13, IOCON_FUNC1 | slew_mask | IOCON_MODE_INACT | IOCON_DIGITAL_EN); /* ENET_TX_EN */
    IOCON_PinMuxSet(IOCON, 4, 14, IOCON_FUNC1 | slew_mask | IOCON_MODE_INACT | IOCON_DIGITAL_EN); /* ENET_RX_CLK */
    IOCON_PinMuxSet(IOCON, 4, 15, IOCON_FUNC1 | slew_mask | IOCON_MODE_INACT | IOCON_DIGITAL_EN); /* ENET_MDC */
    IOCON_PinMuxSet(IOCON, 4, 16, IOCON_FUNC1 | slew_mask | IOCON_MODE_INACT | IOCON_DIGITAL_EN); /* ENET_MDIO */
    IOCON_PinMuxSet(IOCON, 0, 17, IOCON_FUNC7 | slew_mask | IOCON_MODE_INACT | IOCON_DIGITAL_EN); /* ENET_TXD1 */

    IOCON_PinMuxSet(IOCON, 2, 26, IOCON_FUNC0 | IOCON_MODE_PULLUP | IOCON_DIGITAL_EN);            /* ENET_PHY_RSTn */
}

void BOARD_InitSdifPins(void)
{
    /* SDIF pin configuration. */
    IOCON_PinMuxSet(IOCON, 2,  5, IOCON_FUNC2 | IOCON_DIGITAL_EN | IOCON_MODE_INACT); /* SD_POW_EN */
    IOCON_PinMuxSet(IOCON, 3, 15, IOCON_FUNC2 | IOCON_DIGITAL_EN | IOCON_MODE_INACT); /* SD_WPn */
    IOCON_PinMuxSet(IOCON, 2, 10, IOCON_FUNC2 | IOCON_DIGITAL_EN | IOCON_MODE_INACT); /* SD_CDn */

    IOCON_PinMuxSet(IOCON, 2,  9, IOCON_FUNC2 | IOCON_PIO_SLEW_MASK | IOCON_DIGITAL_EN | IOCON_MODE_INACT ); /* SD_D3 */
    IOCON_PinMuxSet(IOCON, 2,  8, IOCON_FUNC2 | IOCON_PIO_SLEW_MASK | IOCON_DIGITAL_EN | IOCON_MODE_INACT ); /* SD_D2 */
    IOCON_PinMuxSet(IOCON, 2,  7, IOCON_FUNC2 | IOCON_PIO_SLEW_MASK | IOCON_DIGITAL_EN | IOCON_MODE_INACT ); /* SD_D1 */
    IOCON_PinMuxSet(IOCON, 2,  6, IOCON_FUNC2 | IOCON_PIO_SLEW_MASK | IOCON_DIGITAL_EN | IOCON_MODE_INACT ); /* SD_D0 */
    IOCON_PinMuxSet(IOCON, 2,  3, IOCON_FUNC2 | IOCON_PIO_SLEW_MASK | IOCON_DIGITAL_EN | IOCON_MODE_INACT ); /* SD_CLK */
    IOCON_PinMuxSet(IOCON, 2,  4, IOCON_FUNC2 | IOCON_PIO_SLEW_MASK | IOCON_DIGITAL_EN | IOCON_MODE_INACT ); /* SD_CMD */

    /* It is important to config the other 4 data bus IO even if you don't use 8bit mode, otherwise 4bit mode will not work properly. */
    IOCON_PinMuxSet(IOCON, 4, 29, IOCON_FUNC2 | IOCON_PIO_SLEW_MASK | IOCON_DIGITAL_EN | IOCON_MODE_PULLUP); /* SD_D4 */
    IOCON_PinMuxSet(IOCON, 4, 30, IOCON_FUNC2 | IOCON_PIO_SLEW_MASK | IOCON_DIGITAL_EN | IOCON_MODE_PULLUP); /* SD_D5 */
    IOCON_PinMuxSet(IOCON, 4, 31, IOCON_FUNC2 | IOCON_PIO_SLEW_MASK | IOCON_DIGITAL_EN | IOCON_MODE_PULLUP); /* SD_D6 */
    IOCON_PinMuxSet(IOCON, 5,  0, IOCON_FUNC2 | IOCON_PIO_SLEW_MASK | IOCON_DIGITAL_EN | IOCON_MODE_PULLUP); /* SD_D7 */
}
